﻿using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.ViewModels;
using SecuritySystemDataModels.Models;

namespace SecuritySystemListImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        public string Name { get; private set; } = string.Empty;
        public string Address { get; private set; } = string.Empty;
        public DateTime OpeningDate { get; private set; }
        public int MaxSecuresCount { get; private set; }
        public Dictionary<int, (ISecureModel, int)> ShopSecures { get; private set; } = new();
        public static Shop? Create(ShopBindingModel model)
        {
            if (model == null)
            {
                return null;
            }

            return new Shop()
            {
                Id = model.Id,
                Name = model.Name,
                Address = model.Address,
                OpeningDate = model.OpeningDate,
                MaxSecuresCount = model.MaxSecuresCount,
                ShopSecures = new()
            };
        }

        public void Update(ShopBindingModel? model)
        {
            if (model == null)
            {
                return;
            }

            Name = model.Name;
            Address = model.Address;
            OpeningDate = model.OpeningDate;
            MaxSecuresCount = model.MaxSecuresCount;
            ShopSecures = model.ShopSecures;
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            Name = Name,
            Address = Address,
            OpeningDate = OpeningDate,
            MaxSecuresCount = MaxSecuresCount,
            ShopSecures = ShopSecures
        };
    }
}
