﻿using Microsoft.Extensions.Logging;
using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.BusinessLogicsContracts;

namespace SecuritySystemView.Shop
{
    public partial class FormShopSell : Form
    {
        private readonly IShopLogic _shopLogic;
        private readonly ISecureLogic _secureLogic;
        private readonly ILogger _logger;
        public FormShopSell(ILogger<FormShopSell> logger, IShopLogic shopLogic, ISecureLogic secureLogic)
        {
            InitializeComponent();
            _logger = logger;
            _shopLogic = shopLogic;
            _secureLogic = secureLogic;
        }

        private void FormShopSell_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Загрузка продукции для продажи");
            try
            {
                var list = _secureLogic.ReadList(null);
                if (list != null)
                {
                    comboBoxSecure.DisplayMember = "SecureName";
                    comboBoxSecure.ValueMember = "Id";
                    comboBoxSecure.DataSource = list;
                    comboBoxSecure.SelectedItem = null;
                }

            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка продукции");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonSell_Click(object sender, EventArgs e)
        {
            if (numericUpDownCount.Value < 1)
            {
                MessageBox.Show("Количество продукта должно быть больше нуля", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            if (comboBoxSecure.SelectedValue == null)
            {
                MessageBox.Show("Выберите продукт", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            _logger.LogInformation("Создание продажи");
            try
            {
                var operationResult = _shopLogic.SellSecures(
                    new SecureBindingModel
                    {
                        Id = Convert.ToInt32(comboBoxSecure.SelectedValue)
                    },
                    Convert.ToInt32(numericUpDownCount.Value)
                );
                if (!operationResult)
                {
                    throw new Exception("Ошибка при создании продажи. Дополнительная информация в логах.");
                }
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания продажи");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
