﻿using Microsoft.Extensions.Logging;
using SecuritySystemContracts.BindingModels;
using SecuritySystemContracts.BusinessLogicsContracts;
using SecuritySystemContracts.SearchModels;

namespace SecuritySystemView
{
    public partial class FormShopSupply : Form
    {
        private readonly ILogger _logger;
        private readonly ISecureLogic _logicSecure;
        private readonly IShopLogic _logicShop;

        public FormShopSupply(ILogger<FormShopSupply> logger, ISecureLogic logicSecure, IShopLogic logicShop)
        {
            InitializeComponent();
            _logger = logger;
            _logicSecure = logicSecure;
            _logicShop = logicShop;
        }

        private void FormShopSupply_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Загрузка secure для пополнения");

            try
            {
                var list = _logicSecure.ReadList(null);
                if (list != null)
                {
                    comboBoxSecure.DisplayMember = "SecureName";
                    comboBoxSecure.ValueMember = "Id";
                    comboBoxSecure.DataSource = list;
                    comboBoxSecure.SelectedItem = null;
                }

            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка secure");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

            _logger.LogInformation("Загрузка магазинов для пополнения");

            try
            {
                var list = _logicShop.ReadList(null);
                if (list != null)
                {
                    comboBoxShop.DisplayMember = "Name";
                    comboBoxShop.ValueMember = "Id";
                    comboBoxShop.DataSource = list;
                    comboBoxShop.SelectedItem = null;
                }

            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка магазинов");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonSupply_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCount.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (comboBoxSecure.SelectedValue == null)
            {
                MessageBox.Show("Выберите secure", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (comboBoxShop.SelectedValue == null)
            {
                MessageBox.Show("Выберите магазин", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Создание поставки");

            try
            {
                var operationResult = _logicShop.SupplySecures(
                    new ShopSearchModel { Id = Convert.ToInt32(comboBoxShop.SelectedValue), Name = comboBoxShop.Text },
                    new SecureBindingModel { Id = Convert.ToInt32(comboBoxSecure.SelectedValue), SecureName = comboBoxSecure.Text },
                    Convert.ToInt32(textBoxCount.Text)
                );

                if (!operationResult)
                {
                    throw new Exception("Ошибка при создании поставки. Дополнительная информация в логах.");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания поставки");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void buttonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
